@@ -65,7 +65,11 @@ class DefaultDbtAdaptee(DbtAdaptee):
         out, err = proc.communicate()
         # Exit code ref: https://docs.getdbt.com/reference/exit-codes
         if proc.returncode == 2:
-            raise DbtInvocationError()
+            message = "The dbt invocation completed with an error. Exit code: 2"
+            if out:
+                message = f"{message}\n{out.decode()}"
+            raise DbtInvocationError(message)
+
         if out:
             return out.decode()
 
@@ -232,7 +236,8 @@ class DbtAdapter:
 
         run_results = self.adaptee.get_run_results()
 
-        output = {}
+        output = []
+        compatible_output = {}
         unique_tests = {}
 
         for result in run_results.get('results', []):
@@ -273,18 +278,36 @@ class DbtAdapter:
 
             parent_table = f'{schema}.{table_name}' if schema and schema != default_schema else table_name
 
-            if parent_table not in output:
-                output[parent_table] = dict(columns={}, tests=[])
+            if parent_table not in compatible_output:
+                compatible_output[parent_table] = dict(columns={}, tests=[])
 
-            if column not in output[parent_table]['columns']:
-                output[parent_table]['columns'][column] = []
-            output[parent_table]['columns'][column].append(dict(
+            if column not in compatible_output[parent_table]['columns']:
+                compatible_output[parent_table]['columns'][column] = []
+            compatible_output[parent_table]['columns'][column].append(dict(
+                id=unique_id,
                 name=unique_id,
+                table=parent_table,
+                column=column if column != resource['name'] else None,
                 status='passed' if unique_tests[unique_id]['status'] == 'pass' else 'failed',
+                tags=[],
                 message=unique_tests[unique_id]['message'],
+                display_name=resource['name'],
+                source='dbt'
             ))
 
-        return output
+            output.append(dict(
+                id=unique_id,
+                name=unique_id,
+                table=parent_table,
+                column=column if column != resource['name'] else None,
+                status='passed' if unique_tests[unique_id]['status'] == 'pass' else 'failed',
+                tags=[],
+                message=unique_tests[unique_id]['message'],
+                display_name=resource['name'],
+                source='dbt'
+            ))
+
+        return output, compatible_output
 
     def append_descriptions(self, profile_result, default_schema):
         out = self.adaptee.list_resources(['model', 'source'], 'resource_type,description,name,columns,source_name')
