@@ -5,10 +5,12 @@ from typing import List
 
 import inquirer
 from ruamel import yaml
+from ruamel.yaml import CommentedMap
 
 from piperider_cli.datasource import DATASOURCE_PROVIDERS, DataSource
 from piperider_cli.error import \
     PipeRiderConfigError, \
+    PipeRiderConfigTypeError, \
     PipeRiderInvalidDataSourceError, \
     DbtProjectNotFoundError, \
     DbtProfileNotFoundError, \
@@ -33,12 +35,17 @@ class Configuration(object):
 
     def __init__(self, dataSources: List[DataSource], **kwargs):
         self.dataSources: List[DataSource] = dataSources
+        self.profiler_config = kwargs.get('profiler', {})
+        self.includes = kwargs.get('includes', None)
+        self.excludes = kwargs.get('excludes', None)
         self.tables = kwargs.get('tables', {})
         self.telemetry_id = kwargs.get('telemetry_id', None)
-        if self.telemetry_id is None:
-            self.telemetry_id = uuid.uuid4().hex
         self.report_dir = self._to_report_dir(kwargs.get('report_dir', '.'))
 
+        self._verify_input_config()
+        self.includes = [str(t) for t in self.includes] if self.includes else self.includes
+        self.excludes = [str(t) for t in self.excludes] if self.excludes else self.excludes
+
     def _to_report_dir(self, dirname: str):
         if dirname is None or dirname.strip() == '':
             dirname = '.'
@@ -46,6 +53,24 @@ class Configuration(object):
             return os.path.abspath(os.path.join(os.getcwd(), PIPERIDER_WORKSPACE_NAME, dirname))
         return os.path.abspath(dirname)
 
+    def _verify_input_config(self):
+        if self.profiler_config:
+            limit = self.profiler_config.get('table', {}).get('limit', 0)
+            if not isinstance(limit, int):
+                raise PipeRiderConfigTypeError("profiler 'limit' should be an integer")
+
+            duplicate_rows = self.profiler_config.get('table', {}).get('duplicateRows', False)
+            if not isinstance(duplicate_rows, bool):
+                raise PipeRiderConfigTypeError("profiler 'duplicateRows' should be an boolean")
+
+        if self.includes is not None:
+            if not isinstance(self.includes, List):
+                raise PipeRiderConfigTypeError("'includes' should be a list of tables' name")
+
+        if self.excludes is not None:
+            if not isinstance(self.excludes, List):
+                raise PipeRiderConfigTypeError("'excludes' should be a list of tables' name")
+
     def get_telemetry_id(self):
         return self.telemetry_id
 
@@ -162,7 +187,10 @@ class Configuration(object):
 
         return cls(
             dataSources=data_sources,
+            profiler=config.get('profiler', {}),
             tables=config.get('tables', {}),
+            includes=config.get('includes', None),
+            excludes=config.get('excludes', None),
             telemetry_id=config.get('telemetry', {}).get('id'),
             report_dir=config.get('report_dir', '.')
         )
@@ -173,23 +201,53 @@ class Configuration(object):
         :param path:
         :return:
         """
+
+        if self.telemetry_id is None:
+            self.telemetry_id = uuid.uuid4().hex
+
         config = dict(
             dataSources=[],
-            tables={},
             telemetry=dict(id=self.telemetry_id)
         )
 
         for d in self.dataSources:
-            if d.credential_source == 'config':
-                datasource = dict(name=d.name, type=d.type_name, **d.credential)
-            else:
-                datasource = dict(name=d.name, type=d.type_name)
+            datasource = dict(name=d.name, type=d.type_name)
             if d.args.get('dbt'):
+                # dbt project
                 datasource['dbt'] = d.args.get('dbt')
+            else:
+                # non-dbt project
+                if d.credential_source == 'config':
+                    datasource.update(**d.credential)
+
             config['dataSources'].append(datasource)
 
+        template = '''
+profiler:
+  table:
+    # the maximum row count to profile. (Default unlimited)
+    limit: 1000000
+    duplicateRows: false
+
+The tables to include/exclude
+includes: []
+excludes: []
+
+tables:
+  my-table-name:
+    # description of the table
+    description: "this is a table description"
+    columns:
+      my-col-name:
+        # description of the column
+        description: "this is a column description"\n
+'''
+
+        config_yaml = CommentedMap(config)
+        config_yaml.yaml_set_comment_before_after_key('telemetry', before=template)
+
         with open(path, 'w') as fd:
-            yaml.round_trip_dump(config, fd)
+            yaml.YAML().dump(config_yaml, fd)
 
     def dump_credentials(self, path, after_init_config=False):
         """
