@@ -6,9 +6,8 @@ class PipeRiderError(Exception):
     """ Base class for piperider errors. """
 
     def __init__(self, *args, **kwargs):
-        self.message = args[0] or ''
+        self.message = args[0] if len(args) else ''
         self.hint = kwargs.get('hint') or ''
-        pass
 
     def __str__(self):
         return self.message
@@ -33,14 +32,6 @@ class PipeRiderConfigTypeError(PipeRiderError):
     hint = "Please check your input configuration in config.yml"
 
 
-class PipeRiderCredentialError(PipeRiderError):
-    def __init__(self, name):
-        self.name = name
-        self.message = f"The credential of '{name}' is not configured."
-
-    hint = "Please execute command 'piperider init' to move forward."
-
-
 class PipeRiderCredentialFieldError(PipeRiderError):
     def __init__(self, field, message):
         self.message = message
@@ -109,12 +100,6 @@ class PipeRiderDataBaseEncodingError(PipeRiderError):
                             f'    \'iconv -f {current_encoding} -t {support_encoding} "{file_path}" > "utf8-{os.path.basename(file_path)}"\''
 
 
-class PipeRiderDiagnosticError(PipeRiderError):
-    def __init__(self, check_name, error_msg, hint=None):
-        self.message = f'{check_name}: {error_msg}'
-        self.hint = hint
-
-
 class DbtError(PipeRiderError):
     type = 'dbt'
 
@@ -152,9 +137,8 @@ class DbtProfileInvalidError(DbtError):
 
 
 class DbtInvocationError(DbtError):
-    def __init__(self, exit_code):
-        self.exit_code = exit_code
-        self.message = f"The dbt invocation completed with an error. Exit code: {exit_code}"
+    def __init__(self, message):
+        self.message = message
 
     hint = "Please reference dbt documentation for more information. ref: https://docs.getdbt.com/reference/exit-codes"
 
